@@ -195,7 +195,7 @@ void apoint_write(struct apoint *o, FILE * f)
 	mem_free(str);
 }
 
-struct apoint *apoint_scan(FILE * f, struct tm start, struct tm end,
+char *apoint_scan(FILE * f, struct tm start, struct tm end,
 			   char state, char *note, struct item_filter *filter)
 {
 	char buf[BUFSIZ], *newline;
@@ -203,15 +203,15 @@ struct apoint *apoint_scan(FILE * f, struct tm start, struct tm end,
 	struct apoint *apt = NULL;
 	int cond;
 
-	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
-		!check_date(end.tm_year, end.tm_mon, end.tm_mday) ||
-		!check_time(start.tm_hour, start.tm_min) ||
-		!check_time(end.tm_hour, end.tm_min),
-		_("date error in appointment"));
+	if (!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
+	    !check_date(end.tm_year, end.tm_mon, end.tm_mday) ||
+	    !check_time(start.tm_hour, start.tm_min) ||
+	    !check_time(end.tm_hour, end.tm_min))
+		return _("illegal date in appointment");
 
 	/* Read the appointment description */
 	if (!fgets(buf, sizeof buf, f))
-		return NULL;
+		return _("error in appointment description");
 
 	newline = strchr(buf, '\n');
 	if (newline)
@@ -226,8 +226,8 @@ struct apoint *apoint_scan(FILE * f, struct tm start, struct tm end,
 
 	tstart = mktime(&start);
 	tend = mktime(&end);
-	EXIT_IF(tstart == -1 || tend == -1 || tstart > tend,
-		_("date error in appointment"));
+	if (tstart == -1 || tend == -1 || tstart > tend)
+		return _("date error in appointment");
 
 	/* Filter item. */
 	if (filter) {
@@ -255,8 +255,7 @@ struct apoint *apoint_scan(FILE * f, struct tm start, struct tm end,
 	}
 	if (!apt)
 		apt = apoint_new(buf, note, tstart, tend - tstart, state);
-
-	return apt;
+	return NULL;
 }
 
 void apoint_delete(struct apoint *apt)
