@@ -49,11 +49,12 @@ int count, reg;
  */
 static void do_storage(int day_changed)
 {
+	int pre_sel;
 	/*
 	 * Save the selected item before rebuilding the day vector -
-	 * unless already set.
+	 * unless a preselection is already set.
 	 */
-	if (!day_check_sel_data())
+	if (!(pre_sel = day_check_sel_data()))
 		day_set_sel_data(ui_day_get_sel());
 
 	if (!day_changed)
@@ -64,7 +65,7 @@ static void do_storage(int day_changed)
 	/* The APP listbox. */
 	ui_day_load_items();
 
-	if (day_changed)
+	if (day_changed && !pre_sel)
 		ui_day_sel_reset();
 	else
 		ui_day_find_sel();
@@ -556,7 +557,8 @@ static inline void key_generic_cmd(void)
 	int valid = 0, force = 0, ret;
 	char *error_msg;
 
-	status_mesg(_("Command: [ h(elp) | w(rite)(!) | q(uit)(!) | wq(!) ]"), "");
+	status_mesg(_("Command: "
+		"[ h(elp) | w(rite)(!) | q(uit)(!) | wq(!) | n(ext) | p(rev) ]"), "");
 	if (getstring(win[STA].p, cmd, BUFSIZ, 0, 1) != GETSTRING_VALID)
 		goto cleanup;
 	cmd_name = strtok(cmd, " ");
@@ -608,6 +610,96 @@ static inline void key_generic_cmd(void)
 		valid = 1;
 	}
 
+	if (!strcmp(cmd_name, "next") || !strcmp(cmd_name, "n")) {
+		struct day_item *item;
+		time_t day, next;
+		struct recur_apoint *rapt;
+		struct recur_event *rev;
+		int more = 0;
+
+		if (wins_slctd() != APP) {
+			error_msg =
+				_("Select a repeating item in the appointments panel.");
+			warnbox(error_msg);
+			goto cleanup;
+		}
+		item = ui_day_get_sel();
+		/*
+		 * The selected day need not be the (item) start day
+		 * for multi-day occurrences.
+		 */
+		day = DAY(item->start);
+		if (item->type == RECUR_EVNT) {
+			rev = item->item.rev;
+			more = recur_next_occurrence(rev->day, -1, rev->rpt, &rev->exc,
+						     day, &next);
+		} else if (item->type == RECUR_APPT) {
+			rapt = item->item.rapt;
+			more = recur_next_occurrence(rapt->start, rapt->dur, rapt->rpt,
+						     &rapt->exc, day, &next);
+		} else {
+			error_msg = _("Not a repeating item.");
+			warnbox(error_msg);
+			goto cleanup;
+		}
+		if (!more) {
+			error_msg = _("Last occurrence.");
+			warnbox(error_msg);
+			goto cleanup;
+		}
+		item->order = next;
+		ui_calendar_set_slctd_day(sec2date(next));
+		day_set_sel_data(item);
+		do_storage(1);
+
+		valid = 1;
+	}
+	if (!strcmp(cmd_name, "prev") || !strcmp(cmd_name, "p")) {
+		struct day_item *item;
+		time_t day, prev;
+		struct recur_apoint *rapt;
+		struct recur_event *rev;
+		int more = 0;
+
+		if (wins_slctd() != APP) {
+			error_msg = _("Select a repeating item in the"
+				" appointments panel.");
+			warnbox(error_msg);
+			goto cleanup;
+		}
+		item = ui_day_get_sel();
+		/*
+		 * The selected day need not be the (item) start day
+		 * for multi-day occurrences.
+		 */
+		day = DAY(item->start);
+		if (item->type == RECUR_EVNT) {
+			rev = item->item.rev;
+			more = recur_prev_occurrence(rev->day, -1, rev->rpt,
+						     &rev->exc, day, &prev);
+		} else if (item->type == RECUR_APPT) {
+			rapt = item->item.rapt;
+			more = recur_prev_occurrence(rapt->start, rapt->dur,
+						     rapt->rpt, &rapt->exc,
+						     day, &prev);
+		} else {
+			error_msg = _("Not a repeating item.");
+			warnbox(error_msg);
+			goto cleanup;
+		}
+		if (!more) {
+			error_msg = _("First occurrence.");
+			warnbox(error_msg);
+			goto cleanup;
+		}
+		item->order = prev;
+		ui_calendar_set_slctd_day(sec2date(prev));
+		day_set_sel_data(item);
+		do_storage(1);
+
+		valid = 1;
+	}
+
 	if (!valid) {
 		asprintf(&error_msg, _("No such command: %s"), cmd);
 		status_mesg(error_msg, "");
@@ -628,8 +720,6 @@ cleanup:
  */
 int main(int argc, char **argv)
 {
-	int no_data_file = 1;
-
 #if ENABLE_NLS
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -644,7 +734,7 @@ int main(int argc, char **argv)
 		/* Non-interactive mode. */
 		exit_calcurse(EXIT_SUCCESS);
 	} else {
-		no_data_file = io_check_data_files();
+		io_check_data_files();
 		dmon_stop();
 		io_set_lock();
 	}
@@ -714,10 +804,6 @@ int main(int argc, char **argv)
 	 * implicitly calling wrefresh() later (causing ncurses race conditions).
 	 */
 	wins_wrefresh(win[KEY].p);
-	if (show_dialogs()) {
-		wins_update(FLAG_ALL);
-		io_startup_screen(no_data_file);
-	}
 	ui_calendar_monthly_view_cache_set_invalid();
 	do_storage(1);
 	ui_todo_load_items();
