@@ -142,12 +142,20 @@
  */
 #define DAYINSEC        (DAYINMIN * MININSEC)
 #define NEXTDAY(date)	date_sec_change((date), 0, 1)
+#define PREVDAY(date)	date_sec_change((date), 0, -1)
 #define DAYLEN(date)	(NEXTDAY(date) - (date))
 #define ENDOFDAY(date)	(NEXTDAY(date) - 1)
 #define HOURINSEC       (HOURINMIN * MININSEC)
+#define DAY(date)	(update_time_in_date(date, 0, 0))
 
 /* Calendar window. */
 #define CALHEIGHT       8
+/*
+ * Week day numbering (0, 1,..., 6) which depends on the first day of the week.
+ * The argument (d) is the "Sunday"-numbering of member tm_wday in struct tm.
+ */
+#define WDAY(d) \
+    (ui_calendar_week_begins_on_monday() ? ((d ? d : WEEKINDAYS) - 1) : d)
 
 /* Key definitions. */
 #define CTRLVAL         0x1F
@@ -285,7 +293,6 @@ struct conf {
 	unsigned confirm_delete;
 	enum win default_panel;
 	unsigned compact_panels;
-	unsigned system_dialogs;
 	unsigned multiple_days;
 	unsigned header_line;
 	unsigned event_separator;
@@ -379,40 +386,57 @@ struct excp {
 };
 
 enum recur_type {
-	RECUR_NO,
 	RECUR_DAILY,
 	RECUR_WEEKLY,
 	RECUR_MONTHLY,
 	RECUR_YEARLY,
-	RECUR_TYPES
+	NBRECUR
 };
 
-/* To describe an item's repetition. */
+/*
+ * Recurrence rule according to RFC5545; used
+ * - in each recurrent appointment/event instance
+ * - in passing parameters as a single function argument
+ */
 struct rpt {
-	enum recur_type type;	/* repetition type */
-	int freq;		/* repetition frequency */
-	time_t until;		/* ending date for repeated event */
+	enum recur_type type;	/* FREQ */
+	int freq;		/* INTERVAL */
+	time_t until;		/* UNTIL */
+	llist_t bymonth;	/* BYMONTH list */
+	llist_t bywday;		/* BY(WEEK)DAY list */
+	llist_t bymonthday;	/* BYMONTHDAY list */
+	llist_t exc;		/* EXDATE's */
 };
 
+/* Types of integers in rrule lists. */
+typedef enum {
+	BYMONTH,
+	BYDAY_W,
+	BYDAY_M,
+	BYDAY_Y,
+	BYMONTHDAY,
+	NOLL
+} int_list_t;
+
 /* Recurrent appointment definition. */
 struct recur_apoint {
-	struct rpt *rpt;	/* information about repetition */
-	llist_t exc;		/* days when the item should not be repeated */
-	time_t start;		/* beggining of the appointment */
-	long dur;		/* duration of the appointment */
-	char state;		/* 8 bits to store item state */
-	char *mesg;		/* appointment description */
-	char *note;		/* note attached to appointment */
+	struct rpt *rpt;	/* recurrence rule */
+	llist_t exc;		/* recurrence exceptions (NOT rpt->exc) */
+	time_t start;		/* start time */
+	long dur;		/* duration */
+	char state;		/* item state */
+	char *mesg;		/* description */
+	char *note;		/* attached note */
 };
 
-/* Reccurent event definition. */
+/* Recurrent event definition. */
 struct recur_event {
-	struct rpt *rpt;	/* information about repetition */
-	llist_t exc;		/* days when the item should not be repeated */
+	struct rpt *rpt;	/* recurrence rule */
+	llist_t exc;		/* recurrence exceptions (NOT rpt->exc) */
 	int id;			/* event type */
-	time_t day;		/* day at which event occurs */
-	char *mesg;		/* event description */
-	char *note;		/* note attached to event */
+	time_t day;		/* day of the event */
+	char *mesg;		/* description */
+	char *note;		/* attached note */
 };
 
 /* Generic pointer data type for appointments and events. */
@@ -488,7 +512,7 @@ struct notify_app {
 
 struct io_file {
 	FILE *fd;
-	char name[BUFSIZ];
+	char *name;
 };
 
 /* Available keys. */
@@ -647,7 +671,6 @@ struct nbar {
 	char datefmt[BUFSIZ];	/* format for displaying date */
 	char timefmt[BUFSIZ];	/* format for displaying time */
 	char cmd[BUFSIZ];	/* notification command */
-	const char *shell;	/* user shell to launch notif. cmd */
 	unsigned notify_all;	/* notify all appointments */
 	pthread_mutex_t mutex;
 };
@@ -762,7 +785,7 @@ void apoint_sec2str(struct apoint *, time_t, char *, char *);
 char *apoint_tostr(struct apoint *);
 char *apoint_hash(struct apoint *);
 void apoint_write(struct apoint *, FILE *);
-struct apoint *apoint_scan(FILE *, struct tm, struct tm, char, char *,
+char *apoint_scan(FILE *, struct tm, struct tm, char, char *,
 			   struct item_filter *);
 void apoint_delete(struct apoint *);
 struct notify_app *apoint_check_next(struct notify_app *, time_t);
@@ -860,7 +883,7 @@ unsigned event_inday(struct event *, time_t *);
 char *event_tostr(struct event *);
 char *event_hash(struct event *);
 void event_write(struct event *, FILE *);
-struct event *event_scan(FILE *, struct tm, int, char *, struct item_filter *);
+char *event_scan(FILE *, struct tm, int, char *, struct item_filter *);
 void event_delete(struct event *);
 void event_paste_item(struct event *, time_t);
 int event_dummy(struct day_item *);
@@ -901,9 +924,8 @@ unsigned io_dir_exists(const char *);
 unsigned io_file_exists(const char *);
 int io_check_file(const char *);
 int io_check_data_files(void);
-void io_startup_screen(int);
 void io_export_data(enum export_type, int);
-void io_import_data(enum import_type, char *, const char *, const char *,
+int io_import_data(enum import_type, char *, const char *, const char *,
 		    const char *, const char *, const char *);
 struct io_file *io_log_init(void);
 void io_log_print(struct io_file *, int, const char *);
@@ -915,7 +937,6 @@ void io_set_lock(void);
 unsigned io_dump_pid(char *);
 unsigned io_get_pid(char *);
 int io_files_equal(const char *, const char *);
-int io_file_is_empty(char *);
 int io_file_cp(const char *, const char *);
 void io_unset_modified(void);
 void io_set_modified(void);
@@ -998,6 +1019,7 @@ void edit_note(char **, const char *);
 void view_note(const char *, const char *);
 void erase_note(char **);
 void note_read(char *, FILE *);
+void note_read_contents(char *, size_t, FILE *);
 void note_gc(void);
 
 /* notify.c */
@@ -1029,7 +1051,11 @@ void pcal_export_data(FILE *);
 /* recur.c */
 extern llist_ts_t recur_alist_p;
 extern llist_t recur_elist;
-int recur_update_exc(llist_t *, char *);
+void recur_free_int_list(llist_t *);
+void recur_int_list_dup(llist_t *, llist_t *);
+void recur_free_exc_list(llist_t *);
+void recur_exc_dup(llist_t *, llist_t *);
+int recur_str2exc(llist_t *, char *);
 char *recur_exc2str(llist_t *);
 struct recur_event *recur_event_dup(struct recur_event *);
 struct recur_apoint *recur_apoint_dup(struct recur_apoint *);
@@ -1042,17 +1068,16 @@ void recur_event_llist_init(void);
 void recur_apoint_llist_free(void);
 void recur_event_llist_free(void);
 struct recur_apoint *recur_apoint_new(char *, char *, time_t, long, char,
-				      int, int, time_t, llist_t *);
-struct recur_event *recur_event_new(char *, char *, time_t, int, int, int,
-				    time_t, llist_t *);
+				      struct rpt *);
+struct recur_event *recur_event_new(char *, char *, time_t, int,
+				     struct rpt *);
 char recur_def2char(enum recur_type);
 int recur_char2def(char);
-struct recur_apoint *recur_apoint_scan(FILE *, struct tm, struct tm,
-				       char, int, struct tm, char *,
-				       llist_t *, char, struct item_filter *);
-struct recur_event *recur_event_scan(FILE *, struct tm, int, char,
-				     int, struct tm, char *, llist_t *,
-				     struct item_filter *);
+char *recur_apoint_scan(FILE *, struct tm, struct tm, char,
+				       char *, struct item_filter *,
+				       struct rpt *);
+char *recur_event_scan(FILE *, struct tm, int, char *,
+				     struct item_filter *, struct rpt *);
 char *recur_apoint_tostr(struct recur_apoint *);
 char *recur_apoint_hash(struct recur_apoint *);
 void recur_apoint_write(struct recur_apoint *, FILE *);
@@ -1060,22 +1085,29 @@ char *recur_event_tostr(struct recur_event *);
 char *recur_event_hash(struct recur_event *);
 void recur_event_write(struct recur_event *, FILE *);
 void recur_save_data(FILE *);
-unsigned recur_item_find_occurrence(time_t, long, llist_t *, int,
-				    int, time_t, time_t, time_t *);
+unsigned recur_item_find_occurrence(time_t, long, struct rpt *, llist_t *,
+				    time_t, time_t *);
 unsigned recur_apoint_find_occurrence(struct recur_apoint *, time_t, time_t *);
 unsigned recur_event_find_occurrence(struct recur_event *, time_t, time_t *);
-unsigned recur_item_inday(time_t, long, llist_t *, int, int, time_t, time_t);
+unsigned recur_item_inday(time_t, long, struct rpt *, llist_t *, time_t);
 unsigned recur_apoint_inday(struct recur_apoint *, time_t *);
 unsigned recur_event_inday(struct recur_event *, time_t *);
 void recur_event_add_exc(struct recur_event *, time_t);
 void recur_apoint_add_exc(struct recur_apoint *, time_t);
 void recur_event_erase(struct recur_event *);
 void recur_apoint_erase(struct recur_apoint *);
+void recur_bymonth(llist_t *, FILE *);
+void recur_bywday(enum recur_type, llist_t *, FILE *);
+void recur_bymonthday(llist_t *, FILE *);
 void recur_exc_scan(llist_t *, FILE *);
 void recur_apoint_check_next(struct notify_app *, time_t, time_t);
 void recur_apoint_switch_notify(struct recur_apoint *);
 void recur_event_paste_item(struct recur_event *, time_t);
 void recur_apoint_paste_item(struct recur_apoint *, time_t);
+int recur_next_occurrence(time_t, long, struct rpt *, llist_t *, time_t, time_t *);
+int recur_nth_occurrence(time_t, long, struct rpt *, llist_t *, int, time_t *);
+int recur_prev_occurrence(time_t, long, struct rpt *, llist_t *, time_t, time_t *);
+
 
 /* sigs.c */
 void sigs_init(void);
@@ -1188,7 +1220,7 @@ int get_item_min(time_t);
 struct tm date2tm(struct date, unsigned, unsigned);
 time_t date2sec(struct date, unsigned, unsigned);
 struct date sec2date(time_t);
-time_t utcdate2sec(struct date, unsigned, unsigned);
+time_t tzdate2sec(struct date, unsigned, unsigned, char *);
 int date_cmp(struct date *, struct date *);
 int date_cmp_day(time_t, time_t);
 char *date_sec2date_str(time_t, const char *);
@@ -1214,13 +1246,13 @@ int check_sec(time_t *);
 int check_time(unsigned, unsigned);
 int parse_time(const char *, unsigned *, unsigned *);
 int parse_duration(const char *, unsigned *, time_t);
-int parse_date_duration(const char *, unsigned *, time_t);
+int parse_date_increment(const char *, unsigned *, time_t);
 int parse_datetime(const char *, time_t *, time_t);
 void file_close(FILE *, const char *);
 void psleep(unsigned);
-int fork_exec(int *, int *, const char *, const char *const *);
-int shell_exec(int *, int *, const char *, const char *const *);
-int child_wait(int *, int *, int);
+int fork_exec(int *, int *, int *, int, const char *, const char *const *);
+int shell_exec(int *, int *, int *, int, const char *, const char *const *);
+int child_wait(int *, int *, int *, int);
 void press_any_key(void);
 void print_apoint(const char *, time_t, struct apoint *);
 void print_event(const char *, time_t, struct event *);
@@ -1232,9 +1264,12 @@ int asprintf(char **, const char *, ...);
 int starts_with(const char *, const char *);
 int starts_with_ci(const char *, const char *);
 int hash_matches(const char *, const char *);
-int show_dialogs(void);
 long overflow_add(long, long, long *);
 long overflow_mul(long, long, long *);
+time_t next_wday(time_t, int);
+int wday_per_year(int, int);
+int wday_per_month(int, int, int);
+char *day_ins(char **, time_t);
 
 /* vars.c */
 extern int col, row;
