@@ -100,7 +100,7 @@ int day_sel_index(void)
 	/* If still not found, stay on the same day. */
 	VECTOR_FOREACH(&day_items, i) {
 		p = VECTOR_NTH(&day_items, i);
-		if (p->order == update_time_in_date(sel_data.order, 0, 0))
+		if (p->order == DAY(sel_data.order))
 			return i;
 	}
 	return -1;
@@ -360,7 +360,7 @@ static int day_store_recur_events(time_t date)
 		p.rev = rev;
 		time_t occurrence;
 		if (recur_event_find_occurrence(rev, date, &occurrence)) {
-			day_add_item(RECUR_EVNT, rev->day, occurrence, p);
+			day_add_item(RECUR_EVNT, occurrence, occurrence, p);
 			e_nb++;
 		}
 	}
@@ -581,9 +581,27 @@ void day_write_stdout(time_t date, const char *fmt_apt, const char *fmt_rapt,
 /* Display an item inside a popup window. */
 void day_popup_item(struct day_item *day)
 {
+	const char *note_heading = _("Note:");
+	size_t note_size = 3500;
+
 	if (day->type == EVNT || day->type == RECUR_EVNT) {
-		item_in_popup(NULL, NULL, day_item_get_mesg(day),
-			      _("Event:"));
+		if (day_item_get_note(day)) {
+			char note[note_size];
+			char *notepath, *msg;
+			FILE *fp;
+
+			asprintf(&notepath, "%s%s", path_notes, day_item_get_note(day));
+			fp = fopen(notepath, "r");
+			note_read_contents(note, note_size, fp);
+			fclose(fp);
+			mem_free(notepath);
+
+			asprintf(&msg, "%s\n\n%s\n%s", day_item_get_mesg(day), note_heading, note);
+			item_in_popup(NULL, NULL, msg, _("Event:"));
+			mem_free(msg);
+		} else {
+			item_in_popup(NULL, NULL, day_item_get_mesg(day), _("Event:"));
+		}
 	} else if (day->type == APPT || day->type == RECUR_APPT) {
 		char a_st[100], a_end[100];
 
@@ -593,8 +611,24 @@ void day_popup_item(struct day_item *day)
 		apt_tmp.start = day->start;
 		apt_tmp.dur = day_item_get_duration(day);
 		apoint_sec2str(&apt_tmp, ui_day_sel_date(), a_st, a_end);
-		item_in_popup(a_st, a_end, day_item_get_mesg(day),
-			      _("Appointment:"));
+
+		if (day_item_get_note(day)) {
+			char note[note_size];
+			char *notepath, *msg;
+			FILE *fp;
+
+			asprintf(&notepath, "%s%s", path_notes, day_item_get_note(day));
+			fp = fopen(notepath, "r");
+			note_read_contents(note, note_size, fp);
+			fclose(fp);
+			mem_free(notepath);
+
+			asprintf(&msg, "%s\n\n%s\n%s", day_item_get_mesg(day), note_heading, note);
+			item_in_popup(a_st, a_end, msg, _("Appointment:"));
+			mem_free(msg);
+		} else {
+			item_in_popup(a_st, a_end, day_item_get_mesg(day), _("Appointment:"));
+		}
 	} else {
 		EXIT(_("unknown item type"));
 		/* NOTREACHED */
@@ -798,7 +832,7 @@ int day_paste_item(struct day_item *p, time_t date)
 		/* wanted: until = shift + old_until */
 		if (p->item.rapt->rpt->until &&
 		    overflow_add(
-			date - update_time_in_date(p->item.rapt->start, 0, 0),
+			date - DAY(p->item.rapt->start),
 			p->item.rapt->rpt->until,
 			&until)
 		)
