@@ -237,25 +237,25 @@ void io_dump_apts(const char *fmt_apt, const char *fmt_rapt,
 
 	LLIST_FOREACH(&recur_elist, i) {
 		struct recur_event *rev = LLIST_GET_DATA(i);
-		time_t day = update_time_in_date(rev->day, 0, 0);
+		time_t day = DAY(rev->day);
 		print_recur_event(fmt_rev, day, rev);
 	}
 
 	LLIST_TS_FOREACH(&recur_alist_p, i) {
 		struct recur_apoint *rapt = LLIST_GET_DATA(i);
-		time_t day = update_time_in_date(rapt->start, 0, 0);
+		time_t day = DAY(rapt->start);
 		print_recur_apoint(fmt_rapt, day, rapt->start, rapt);
 	}
 
 	LLIST_TS_FOREACH(&alist_p, i) {
 		struct apoint *apt = LLIST_TS_GET_DATA(i);
-		time_t day = update_time_in_date(apt->start, 0, 0);
+		time_t day = DAY(apt->start);
 		print_apoint(fmt_apt, day, apt);
 	}
 
 	LLIST_FOREACH(&eventlist, i) {
 		struct event *ev = LLIST_TS_GET_DATA(i);
-		time_t day = update_time_in_date(ev->day, 0, 0);
+		time_t day = DAY(ev->day);
 		print_event(fmt_ev, day, ev);
 	}
 }
@@ -553,13 +553,13 @@ void io_load_app(struct item_filter *filter)
 	FILE *data_file;
 	int c, is_appointment, is_event, is_recursive;
 	struct tm start, end, until, lt;
-	llist_t exc;
+	struct rpt rpt;
 	time_t t;
 	int id = 0;
-	int freq;
 	char type, state = 0L;
 	char note[MAX_NOTESIZ + 1], *notep;
 	unsigned line = 0;
+	char *scan_error;
 
 	t = time(NULL);
 	localtime_r(&t, &lt);
@@ -572,9 +572,10 @@ void io_load_app(struct item_filter *filter)
 	rewind(data_file);
 
 	for (;;) {
-		LLIST_INIT(&exc);
 		is_appointment = is_event = is_recursive = 0;
 		line++;
+		scan_error = NULL;
+
 		c = getc(data_file);
 		if (c == EOF)
 			break;
@@ -630,102 +631,113 @@ void io_load_app(struct item_filter *filter)
 
 		if (c == '{') {
 			is_recursive = 1;
-			if (fscanf(data_file, " %d%c ", &freq, &type) != 2)
+			if (fscanf(data_file, " %d%c ", &rpt.freq, &type) != 2)
 				io_load_error(path_apts, line,
 					      _("syntax error in item repetition"));
-
+			else
+				rpt.type = recur_char2def(type);
 			c = getc(data_file);
-			if (c == '}') {	/* endless recurrent item */
-				until.tm_year = 0;
-				while ((c = getc(data_file)) == ' ') ;
-				ungetc(c, data_file);
-			} else if (c == '-' && getc(data_file) == '>') {
+			/* Optional until date */
+			if (c == '-' && getc(data_file) == '>') {
 				if (fscanf
 				    (data_file, " %d / %d / %d ",
 				     &until.tm_mon, &until.tm_mday,
 				     &until.tm_year) != 3)
 					io_load_error(path_apts, line,
-						      _("syntax error in item repetition"));
-				c = getc(data_file);
-				if (c == '!') {
-					ungetc(c, data_file);
-					recur_exc_scan(&exc, data_file);
-					while ((c =
-						getc(data_file)) == ' ') ;
-					ungetc(c, data_file);
-				} else if (c == '}') {
-					while ((c =
-						getc(data_file)) == ' ') ;
-					ungetc(c, data_file);
-				} else {
+						      _("syntax error in until date"));
+				if (!check_date(until.tm_year, until.tm_mon,
+						until.tm_mday))
 					io_load_error(path_apts, line,
-						      _("syntax error in item repetition"));
-				}
-			} else if (c == '!') {	/* endless item with exceptions */
+						      _("until date error"));
+				until.tm_hour = 0;
+				until.tm_min = 0;
+				until.tm_sec = 0;
+				until.tm_isdst = -1;
+				until.tm_year -= 1900;
+				until.tm_mon--;
+				rpt.until = mktime(&until);
+				c = getc(data_file);
+			} else
+				rpt.until = 0;
+			/* Optional bymonthday list */
+			if (c == 'd') {
+				if (rpt.type == RECUR_WEEKLY)
+					io_load_error(path_apts, line,
+						      _("BYMONTHDAY illegal with WEEKLY"));
 				ungetc(c, data_file);
-				recur_exc_scan(&exc, data_file);
-				while ((c = getc(data_file)) == ' ') ;
+				recur_bymonthday(&rpt.bymonthday, data_file);
+				c = getc(data_file);
+			} else
+				LLIST_INIT(&rpt.bymonthday);
+			/* Optional bywday list */
+			if (c == 'w') {
 				ungetc(c, data_file);
-				until.tm_year = 0;
-			} else {
+				recur_bywday(rpt.type, &rpt.bywday, data_file);
+				c = getc(data_file);
+			} else
+				LLIST_INIT(&rpt.bywday);
+			/* Optional bymonth list */
+			if (c == 'm') {
+				ungetc(c, data_file);
+				recur_bymonth(&rpt.bymonth, data_file);
+				c = getc(data_file);
+			} else
+				LLIST_INIT(&rpt.bymonth);
+			/* Optional exception dates */
+			if (c == '!') {
+				ungetc(c, data_file);
+				recur_exc_scan(&rpt.exc, data_file);
+				c = getc(data_file);
+			} else
+				LLIST_INIT(&rpt.exc);
+			/* End of recurrence rule */
+			if (c != '}')
 				io_load_error(path_apts, line,
-					      _("wrong format in the appointment or event"));
-				/* NOTREACHED */
-			}
-		} else {
-			ungetc(c, data_file);
+					      _("missing end of recurrence"));
+			while ((c = getc(data_file)) == ' ') ;
 		}
 
 		/* Check if a note is attached to the item. */
-		c = getc(data_file);
 		if (c == '>') {
 			note_read(note, data_file);
+			c = getc(data_file);
 			notep = note;
-		} else {
+		} else
 			notep = NULL;
-			ungetc(c, data_file);
-		}
 
 		/*
 		 * Last: read the item description and load it into its
 		 * corresponding linked list, depending on the item type.
 		 */
 		if (is_appointment) {
-			c = getc(data_file);
-			if (c == '!') {
+			if (c == '!')
 				state |= APOINT_NOTIFY;
-				while ((c = getc(data_file)) == ' ') ;
-				ungetc(c, data_file);
-			} else if (c == '|') {
+			else if (c == '|')
 				state = 0L;
-				while ((c = getc(data_file)) == ' ') ;
-				ungetc(c, data_file);
-			} else {
+			else
 				io_load_error(path_apts, line,
-					      _("syntax error in item repetition"));
-			}
-			if (is_recursive) {
-				recur_apoint_scan(data_file, start, end,
-						  type, freq, until, notep,
-						  &exc, state, filter);
-			} else {
-				apoint_scan(data_file, start, end, state,
+					      _("syntax error in item state"));
+
+			if (is_recursive)
+				scan_error = recur_apoint_scan(data_file, start, end, state,
+						  notep, filter, &rpt);
+			else
+				scan_error = apoint_scan(data_file, start, end, state,
 					    notep, filter);
-			}
 		} else if (is_event) {
-			if (is_recursive) {
-				recur_event_scan(data_file, start, id,
-						 type, freq, until, notep,
-						 &exc, filter);
-			} else {
-				event_scan(data_file, start, id, notep,
-					   filter);
-			}
+			ungetc(c, data_file);
+			if (is_recursive)
+				scan_error = recur_event_scan(data_file, start, id, notep,
+						 filter, &rpt);
+			else
+				scan_error = event_scan(data_file, start, id, notep, filter);
 		} else {
 			io_load_error(path_apts, line,
 				      _("wrong format in the appointment or event"));
 			/* NOTREACHED */
 		}
+		if (scan_error)
+			io_load_error(path_apts, line, scan_error);
 	}
 	file_close(data_file, __FILE_POS__);
 }
@@ -1195,21 +1207,6 @@ int io_check_data_files(void)
 	return missing;
 }
 
-/* Draw the startup screen */
-void io_startup_screen(int no_data_file)
-{
-	const char *enter = _("Press [ENTER] to continue");
-
-	if (no_data_file)
-		status_mesg(_("Welcome to Calcurse. Missing data files were created."),
-			    enter);
-	else
-		status_mesg(_("Data files found. Data will be loaded now."),
-			    enter);
-
-	keys_wait_for_any_key(win[KEY].p);
-}
-
 /* Export calcurse data. */
 void io_export_data(enum export_type type, int export_uid)
 {
@@ -1240,7 +1237,7 @@ void io_export_data(enum export_type type, int export_uid)
 	else if (type == IO_EXPORT_PCAL)
 		pcal_export_data(stream);
 
-	if (show_dialogs() && ui_mode == UI_CURSES) {
+	if (!quiet && ui_mode == UI_CURSES) {
 		fclose(stream);
 		status_mesg(success, enter);
 		keys_wait_for_any_key(win[KEY].p);
@@ -1280,7 +1277,7 @@ static FILE *get_import_stream(enum import_type type, char **stream_name)
  * A temporary log file is created in /tmp to store the import process report,
  * and is cleared at the end.
  */
-void io_import_data(enum import_type type, char *stream_name,
+int  io_import_data(enum import_type type, char *stream_name,
 		    const char *fmt_ev, const char *fmt_rev,
 		    const char *fmt_apt, const char *fmt_rapt,
 		    const char *fmt_todo)
@@ -1315,7 +1312,7 @@ void io_import_data(enum import_type type, char *stream_name,
 	}
 
 	if (stream == NULL)
-		return;
+		return 0;
 
 	memset(&stats, 0, sizeof stats);
 
@@ -1323,7 +1320,7 @@ void io_import_data(enum import_type type, char *stream_name,
 	if (log == NULL) {
 		if (stream != stdin)
 			file_close(stream, __FILE_POS__);
-		return;
+		return 0;
 	}
 
 	if (type == IO_IMPORT_ICAL)
@@ -1348,7 +1345,7 @@ void io_import_data(enum import_type type, char *stream_name,
 		 stats.todos);
 	asprintf(&stats_str[3], _("%d skipped"), stats.skipped);
 
-	if (ui_mode == UI_CURSES && show_dialogs()) {
+	if (ui_mode == UI_CURSES && !quiet) {
 		char *read, *stat;
 
 		asprintf(&read, proc_report, stats.lines);
@@ -1359,7 +1356,7 @@ void io_import_data(enum import_type type, char *stream_name,
 		mem_free(read);
 		mem_free(stat);
 		keys_wait_for_any_key(win[KEY].p);
-	} else if (ui_mode == UI_CMDLINE && show_dialogs()) {
+	} else if (ui_mode == UI_CMDLINE && !quiet) {
 		printf(proc_report, stats.lines);
 		printf("\n%s / %s / %s / %s\n", stats_str[0], stats_str[1],
 		       stats_str[2], stats_str[3]);
@@ -1380,8 +1377,11 @@ void io_import_data(enum import_type type, char *stream_name,
 	mem_free(stats_str[3]);
 	if (ui_mode == UI_CURSES)
 		mem_free(stream_name);
-	if (!stats.skipped)
+	if (!stats.skipped) {
 		io_log_free(log);
+		return 1;
+	} else
+		return 0;
 }
 
 struct io_file *io_log_init(void)
@@ -1399,7 +1399,7 @@ struct io_file *io_log_init(void)
 		ERROR_MSG(_("Warning: could not create temporary log file, Aborting..."));
 		goto error;
 	}
-	strncpy(log->name, logname, sizeof(log->name));
+	log->name = mem_strdup(logname);
 	log->fd = fopen(log->name, "w");
 	if (log->fd == NULL) {
 		ERROR_MSG(_("Warning: could not open temporary log file, Aborting..."));
@@ -1449,6 +1449,7 @@ void io_log_free(struct io_file *log)
 	EXIT_IF(unlink(log->name) != 0,
 		_("Warning: could not erase temporary log file %s, Aborting..."),
 		log->name);
+	mem_free(log->name);
 	mem_free(log);
 }
 
@@ -1457,7 +1458,8 @@ static void *io_psave_thread(void *arg)
 {
 	int delay = conf.periodic_save;
 	EXIT_IF(delay < 0, _("Invalid delay"));
-	char *mesg = _("Periodic save: data files have changed. Save cancelled.");
+	char *mesg = _("Periodic save cancelled. Data files have changed. "
+		     "Save and merge interactively");
 
 	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 	for (;;) {
@@ -1576,22 +1578,6 @@ unsigned io_get_pid(char *file)
 	return pid;
 }
 
-/*
- * Check whether a file is empty.
- */
-int io_file_is_empty(char *file)
-{
-	FILE *fp;
-	int ret = -1;
-
-	if (file && (fp = fopen(file, "r"))) {
-		ret = (fgetc(fp) == '\n' && fgetc(fp) == EOF) || feof(fp);
-		fclose(fp);
-	}
-
-	return ret;
-}
-
 /*
  * Check whether two files are equal.
  */
