@@ -59,13 +59,9 @@ HTABLE_PROTOTYPE(htp, note_gc_hash)
 char *generate_note(const char *str)
 {
 	char *sha1 = mem_malloc(SHA1_DIGESTLEN * 2 + 1);
-	char *notepath, *s;
+	char *notepath;
 	FILE *fp;
 
-	/* Temporary hack */
-	asprintf(&s, "%s\n", str);
-	str = s;
-
 	sha1_digest(str, sha1);
 	asprintf(&notepath, "%s%s", path_notes, sha1);
 	fp = fopen(notepath, "w");
@@ -74,7 +70,6 @@ char *generate_note(const char *str)
 	fputs(str, fp);
 	file_close(fp, __FILE_POS__);
 
-	mem_free(s);
 	mem_free(notepath);
 	return sha1;
 }
@@ -99,9 +94,7 @@ void edit_note(char **note, const char *editor)
 	const char *arg[] = { editor, tmppath, NULL };
 	wins_launch_external(arg);
 
-	if (io_file_is_empty(tmppath) > 0) {
-		erase_note(note);
-	} else if ((fp = fopen(tmppath, "r"))) {
+	if ((fp = fopen(tmppath, "r"))) {
 		sha1_stream(fp, sha1);
 		fclose(fp);
 		*note = sha1;
@@ -160,6 +153,17 @@ void note_read(char *buffer, FILE * fp)
 	buffer[MAX_NOTESIZ] = '\0';
 }
 
+/* Read the contents of a note file */
+void note_read_contents(char *buffer, size_t buffer_len, FILE * fp)
+{
+	size_t read_count = fread(buffer, 1, buffer_len, fp);
+	if (read_count != buffer_len)
+		buffer[read_count] = '\0';
+	else
+		memcpy(&buffer[buffer_len - 4], "...\0", 4);
+}
+
+
 static void
 note_gc_extract_key(struct note_gc_hash *data, const char **key, int *len)
 {
