@@ -63,7 +63,7 @@ static int layout;
 /*
  * The screen_mutex mutex and wins_refresh(), wins_wrefresh(), wins_doupdate()
  * functions are used to prevent concurrent updates of the screen.
- * It was observed that the display could get screwed up when mulitple threads
+ * It was observed that the display could get screwed up when multiple threads
  * tried to refresh the screen at the same time.
  *
  * Note (2010-03-21):
@@ -593,8 +593,6 @@ void wins_prepare_external(void)
 {
 	if (notify_bar())
 		notify_stop_main_thread();
-	if (conf.periodic_save > 0)
-		io_stop_psave_thread();
 	def_prog_mode();
 	ui_mode = UI_CMDLINE;
 	clear();
@@ -615,8 +613,6 @@ void wins_unprepare_external(void)
 	wins_resize();
 	if (notify_bar())
 		notify_start_main_thread();
-	if (conf.periodic_save > 0)
-		io_start_psave_thread();
 }
 
 /*
@@ -628,8 +624,8 @@ void wins_launch_external(const char *arg[])
 	int pid;
 
 	wins_prepare_external();
-	if ((pid = shell_exec(NULL, NULL, *arg, arg)))
-		child_wait(NULL, NULL, pid);
+	if ((pid = shell_exec(NULL, NULL, NULL, 0, *arg, arg)))
+		child_wait(NULL, NULL, NULL, pid);
 	wins_unprepare_external();
 }
 
