@@ -3,11 +3,11 @@
 . "${TEST_INIT:-./test-init.sh}"
 
 if [ "$1" = 'actual' ]; then
-  mkdir .calcurse || exit 1
-  cp "$DATA_DIR/conf" .calcurse || exit 1
-  "$CALCURSE" -D "$PWD/.calcurse" -i "$DATA_DIR/ical-006.ical"
-  "$CALCURSE" -D "$PWD/.calcurse" -s06/01/2012 -r2
-  rm -rf .calcurse || exit 1
+  tmpdir=$(mktemp -d)
+  cp "$DATA_DIR/conf" "$tmpdir" || exit 1
+  "$CALCURSE" -D "$tmpdir" -i "$DATA_DIR/ical-006.ical"
+  "$CALCURSE" -D "$tmpdir" -s06/01/2012 -r2
+  rm -rf "$tmpdir" || exit 1
 elif [ "$1" = 'expected' ]; then
   cat <<EOD
 Import process report: 0063 lines read
